"""Vera constants."""
DOMAIN = "vera"

CONF_CONTROLLER = "vera_controller_url"

VERA_ID_FORMAT = "{}_{}"

ATTR_CURRENT_POWER_W = "current_power_w"
ATTR_CURRENT_ENERGY_KWH = "current_energy_kwh"

CONTROLLER_DATAS = "controller_datas"
